import 'package:flutter/foundation.dart';
import 'package:ocg_database/src/base.dart';
import 'package:path_provider/path_provider.dart';
import 'package:sembast/sembast.dart';
import 'package:ocg_app/ocg_app.dart';
import 'package:sembast/sembast_io.dart';
import 'package:sembast_web/sembast_web.dart';

class SembastDB<T> extends DB<T> {
  final String path;
  static Database _singleton;
  static StoreRef _storeRef;
  final T Function(Map<String, dynamic>) fromJson;
  final Map<String, dynamic> Function(T object) toJson;

  SembastDB(this.path, this.fromJson, this.toJson);

  bool get isOpened => _singleton != null;

  @override
  Future<AppResponse<bool>> open() async {
    try {
      if (_singleton == null) {
        String path;
        if (kIsWeb) {
          path = "sample.db";
          _singleton = await databaseFactoryWeb.openDatabase(path);
        } else {
          final appDocDir = await getApplicationDocumentsDirectory();
          path = "${appDocDir.path}/$path";
          _singleton = await databaseFactoryIo.openDatabase(path, version: 1);
        }
        _storeRef = StoreRef.main();
      }
      return AppResponse.named(data: true);
    } catch (e) {
      return AppResponse.named(error: "$e");
    }
  }

  @override
  Future<AppResponse<String>> create(object) async {
    try {
      if (!isOpened) {
        await open();
      }
      var json = toJson(object);
      var key = await _storeRef.record(json["id"]).put(_singleton, json);
      return AppResponse.named(data: "$key");
    } catch (e) {
      return AppResponse.named(error: "$e");
    }
  }

  @override
  Future<AppResponse<bool>> delete(String id) async {
    try {
      if (!isOpened) {
        await open();
      }
      await _storeRef.record(id).delete(_singleton);
      return AppResponse.named(data: true);
    } catch (e) {
      return AppResponse.named(error: "$e");
    }
  }

  @override
  Future<AppResponse<List<T>>> getList({int page = 1, int count = 10}) {
    // TODO: implement getList
    throw UnimplementedError();
  }

  @override
  Future<AppResponse<T>> read(String id) async {
    try {
      if (!isOpened) {
        await open();
      }
      var object = await _storeRef.record(id).get(_singleton);
      return AppResponse.named(
          data: fromJson(Map<String, dynamic>.from(object)));
    } catch (e) {
      return AppResponse.named(error: "$e");
    }
  }

  @override
  Future<AppResponse<bool>> update(object) {
    // TODO: implement update
    throw UnimplementedError();
  }

  @override
  bool get isOpen => _singleton != null;
}
